@@ -3,7 +3,7 @@
  * gedit-dialog-goto-line.c
  * This file is part of gedit
  *
- * Copyright (C) 2001-2002 Paolo Maggi 
+ * Copyright (C) 2001 Paolo Maggi 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,12 @@
  */
 
 /*
- * Modified by the gedit Team, 1998-2002. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2001. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  */
 
 #include <glade/glade-xml.h>
-#include <libgnome/libgnome.h>
 
 #include "gedit2.h"
 #include "gedit-mdi.h"
@@ -36,7 +35,6 @@
 #include "gedit-dialogs.h"
 #include "gedit-document.h"
 #include "gedit-view.h"
-#include "gedit-debug.h"
 
 typedef struct _GeditDialogGotoLine GeditDialogGotoLine;
 
@@ -48,59 +46,6 @@ struct _GeditDialogGotoLine {
 
 static void goto_button_pressed (GeditDialogGotoLine * dialog);
 static GeditDialogGotoLine *dialog_goto_line_get_dialog (void);
-static void dialog_destroyed (GtkObject *obj,  void **dialog_pointer);
-
-static void
-dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
-{
-	gedit_debug (DEBUG_SEARCH, "");
-
-	if (dialog_pointer != NULL)
-	{
-		g_free (*dialog_pointer);
-		*dialog_pointer = NULL;
-	}	
-}
-
-static void
-dialog_response_handler (GtkDialog *dlg, gint res_id,  GeditDialogGotoLine *dialog)
-{
-	gedit_debug (DEBUG_SEARCH, "");
-
-	switch (res_id) {
-		case GTK_RESPONSE_OK:
-			goto_button_pressed (dialog);
-			break;
-			
-		default:
-			gtk_widget_destroy (dialog->dialog);
-	}
-}
-
-static void
-entry_insert_text (GtkEditable *editable, const char *text, gint length, gint *position)
-{
-	gunichar c;
-	const gchar *p;
- 	const gchar *end;
-
-	p = text;
-	end = text + length;
-
-	while (p != end) {
-		const gchar *next;
-		next = g_utf8_next_char (p);
-
-		c = g_utf8_get_char (p);
-
-		if (!g_unichar_isdigit (c)) {
-			g_signal_stop_emission_by_name (editable, "insert_text");
-			break;
-		}
-
-		p = next;
-	}
-}
 
 static GeditDialogGotoLine *
 dialog_goto_line_get_dialog (void)
@@ -109,18 +54,9 @@ dialog_goto_line_get_dialog (void)
 	GladeXML *gui;
 	GtkWindow *window;
 	GtkWidget *content;
-	GtkWidget *button;
-
-	gedit_debug (DEBUG_SEARCH, "");
 
 	if (dialog != NULL)
-	{
-		gdk_window_show (dialog->dialog->window);
-		gdk_window_raise (dialog->dialog->window);
-		gtk_widget_grab_focus (dialog->dialog);
-
 		return dialog;
-	}
 
 	gui = glade_xml_new (GEDIT_GLADEDIR "goto-line.glade2",
 			     "goto_line_dialog_content", NULL);
@@ -136,57 +72,35 @@ dialog_goto_line_get_dialog (void)
 
 	dialog = g_new0 (GeditDialogGotoLine, 1);
 
-	dialog->dialog = gtk_dialog_new_with_buttons (_("Go to Line"),
+	dialog->dialog = gtk_dialog_new_with_buttons ("Goto line...",
 						      window,
-						      GTK_DIALOG_DESTROY_WITH_PARENT,
-						      GTK_STOCK_CLOSE,
+						      GTK_DIALOG_MODAL,
+						      GTK_STOCK_CANCEL,
 						      GTK_RESPONSE_CANCEL,
+						      GTK_STOCK_OK,
+						      GTK_RESPONSE_OK,
 						      NULL);
 
 	g_return_val_if_fail (dialog->dialog != NULL, NULL);
 
-	/* Add Goto Line button */
-	button = gedit_button_new_with_stock_image (_("_Go to Line"), GTK_STOCK_JUMP_TO);
-	g_return_val_if_fail (button != NULL, NULL);
-
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-
-	gtk_widget_show (button);
-
-	gtk_dialog_add_action_widget (GTK_DIALOG (dialog->dialog), 
-				      button, GTK_RESPONSE_OK);
-	
 	content = glade_xml_get_widget (gui, "goto_line_dialog_content");
 
 	dialog->entry = glade_xml_get_widget (gui, "entry");
 
 	if (!dialog->entry) {
-		g_warning (
-			_("Could not find the required widgets inside %s."),
-			"goto-line.glade2");
-		g_object_unref (gui);
+		g_print
+		    ("Could not find the required widgets inside goto-line.glade2.\n");
 		return NULL;
 	}
-	
+
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox),
 			    content, FALSE, FALSE, 0);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
 					 GTK_RESPONSE_OK);
 
-	g_signal_connect (G_OBJECT (dialog->entry), "insert_text",
-			  G_CALLBACK (entry_insert_text), NULL);
-
-	g_signal_connect (G_OBJECT (dialog->dialog), "destroy",
-			  G_CALLBACK (dialog_destroyed), &dialog);
-
-	g_signal_connect (G_OBJECT (dialog->dialog), "response",
-			  G_CALLBACK (dialog_response_handler), dialog);
-
 	g_object_unref (gui);
-
-	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
-
+	
 	return dialog;
 }
 
@@ -195,8 +109,7 @@ void
 gedit_dialog_goto_line (void)
 {
 	GeditDialogGotoLine *dialog;
-
-	gedit_debug (DEBUG_SEARCH, "");
+	gint response;
 
 	dialog = dialog_goto_line_get_dialog ();
 	if (dialog == NULL) {
@@ -211,19 +124,25 @@ gedit_dialog_goto_line (void)
 
 	gtk_widget_grab_focus (dialog->entry);
 
-	if (!GTK_WIDGET_VISIBLE (dialog->dialog))
-		gtk_widget_show (dialog->dialog);
+	response = gtk_dialog_run (GTK_DIALOG (dialog->dialog));
+
+	switch (response) {
+		case GTK_RESPONSE_OK:
+			goto_button_pressed (dialog);
+			break;
+
+		default:
+			gtk_widget_hide (dialog->dialog);
+	}
 }
 
 static void
 goto_button_pressed (GeditDialogGotoLine *dialog)
 {
-	const gchar* text;
+	gchar* text;
 	GeditView* active_view;
 	GeditDocument* active_document;
 
-	gedit_debug (DEBUG_SEARCH, "");
-
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
 
@@ -234,10 +153,11 @@ goto_button_pressed (GeditDialogGotoLine *dialog)
 	
 	if (text != NULL && text [0] != 0)
 	{
-		guint line = MAX (atoi (text)- 1, 0);		
+		guint line = MAX (atoi (text), 0);		
 		gedit_document_goto_line (active_document, line);
 		gedit_view_scroll_to_cursor (active_view);
-		gtk_widget_grab_focus (GTK_WIDGET (active_view));
 	}
+
+	gtk_widget_hide (dialog->dialog);
 }
 
