@@ -27,19 +27,11 @@
  * See the ChangeLog files for a list of changes. 
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <glade/glade-xml.h>
-#include <libgnome/gnome-help.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-entry.h>
 
 #include "gedit2.h"
 #include "gedit-mdi.h"
 #include "gedit-utils.h"
-#include "gedit-file.h"
 #include "gedit-dialogs.h"
 
 typedef struct _GeditDialogOpenUri GeditDialogOpenUri;
@@ -80,7 +72,7 @@ dialog_open_uri_get_dialog (void)
 
 	dialog = g_new0 (GeditDialogOpenUri, 1);
 
-	dialog->dialog = gtk_dialog_new_with_buttons (_("Open from URI"),
+	dialog->dialog = gtk_dialog_new_with_buttons ("Open from URI",
 						      window,
 						      GTK_DIALOG_MODAL,
 						      GTK_STOCK_CANCEL,
@@ -112,8 +104,6 @@ dialog_open_uri_get_dialog (void)
 
 	g_object_unref (gui);
 
-	gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
-
 	return dialog;
 }
 
@@ -168,10 +158,6 @@ open_button_pressed (GeditDialogOpenUri * dialog)
 	file_name =
 	    gtk_editable_get_chars (GTK_EDITABLE (dialog->uri), 0, -1);
 
-	gnome_entry_prepend_history (GNOME_ENTRY (dialog->uri_list), 
-				     TRUE,
-				     file_name);
-
 	gtk_widget_hide (dialog->dialog);
 
 	gedit_file_open_single_uri (file_name);
@@ -182,15 +168,5 @@ open_button_pressed (GeditDialogOpenUri * dialog)
 static void
 help_button_pressed (GeditDialogOpenUri * dialog)
 {
-	GError *error = NULL;
-
-	gnome_help_display ("gedit.xml", "gedit-open-from-uri", &error);
-	
-	if (error != NULL)
-	{
-		g_warning (error->message);
-
-		g_error_free (error);
-	}
-
+	/* FIXME */
 }
