@@ -32,7 +32,6 @@
 #include <config.h>
 #endif
 
-#include <string.h>
 #include <gtk/gtk.h>
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -47,6 +46,15 @@
 #include "dialogs/gedit-dialogs.h"
 #include "dialogs/gedit-preferences-dialog.h"
 
+#define TO_BE_IMPLEMENTED	{ GtkWidget *message_dlg; \
+				  message_dlg = gtk_message_dialog_new (	\
+			                          GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),	\
+			                          GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,	\
+			                          GTK_MESSAGE_INFO,	\
+			                          GTK_BUTTONS_OK,	\
+                                   		  _("Not yet implemented."));	\
+	                         gtk_dialog_run (GTK_DIALOG (message_dlg));	\
+  	                         gtk_widget_destroy (message_dlg); }
 
 void 
 gedit_cmd_file_new (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
@@ -79,7 +87,7 @@ gedit_cmd_file_save (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 	if (active_child == NULL)
 		return;
 	
-	gedit_file_save (active_child, TRUE);
+	gedit_file_save (active_child);
 }
 
 void 
@@ -129,6 +137,8 @@ gedit_cmd_file_open_uri (BonoboUIComponent *uic, gpointer user_data, const gchar
 void
 gedit_cmd_file_print (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
+	TO_BE_IMPLEMENTED
+/*		
 	GeditMDIChild *active_child;
 	
 	gedit_debug (DEBUG_COMMANDS, "");
@@ -137,13 +147,16 @@ gedit_cmd_file_print (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 	if (active_child == NULL)
 		return;
 	
+	g_print ("***** Printing in gedit2 is completely broken. Please don't report bugs about it.\n");
 	gedit_print (active_child);
-
+*/
 }
 
 void
 gedit_cmd_file_print_preview (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
+	TO_BE_IMPLEMENTED
+/*
 	GeditMDIChild *active_child;
 	
 	gedit_debug (DEBUG_COMMANDS, "");
@@ -152,7 +165,9 @@ gedit_cmd_file_print_preview (BonoboUIComponent *uic, gpointer user_data, const
 	if (active_child == NULL)
 		return;
 	
+	g_print ("***** Printing in gedit2 is completely broken. Please don't report bugs about it.\n");
 	gedit_print_preview (active_child);
+*/
 }
 
 void 
@@ -166,12 +181,8 @@ gedit_cmd_file_close (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 	
 	if (active_view == NULL)
 		return;
-	
-	gedit_close_x_button_pressed = TRUE;
-	
-	gedit_file_close (active_view);
 
-	gedit_close_x_button_pressed = FALSE;
+	gedit_file_close (active_view);
 }
 
 void 
@@ -179,11 +190,7 @@ gedit_cmd_file_close_all (BonoboUIComponent *uic, gpointer user_data, const gcha
 {
 	gedit_debug (DEBUG_COMMANDS, "");
 
-	gedit_close_x_button_pressed = TRUE;
-	
 	gedit_file_close_all ();
-
-	gedit_close_x_button_pressed = FALSE;
 }
 
 void 
@@ -191,11 +198,7 @@ gedit_cmd_file_exit (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 {
 	gedit_debug (DEBUG_COMMANDS, "");
 
-	gedit_exit_button_pressed = TRUE;
-	
 	gedit_file_exit ();	
-
-	gedit_exit_button_pressed = FALSE;
 }
 
 void 
@@ -212,9 +215,7 @@ gedit_cmd_edit_undo (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 
 	gedit_document_undo (active_document);
 
-	gedit_view_scroll_to_cursor (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	/* TODO: Move the cursor */
 }
 
 void 
@@ -231,9 +232,7 @@ gedit_cmd_edit_redo (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 
 	gedit_document_redo (active_document);
 
-	gedit_view_scroll_to_cursor (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	/* TODO: Move the cursor */
 }
 
 void 
@@ -245,8 +244,6 @@ gedit_cmd_edit_cut (BonoboUIComponent *uic, gpointer user_data, const gchar* ver
 	g_return_if_fail (active_view);
 	
 	gedit_view_cut_clipboard (active_view); 
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
 }
 
 void 
@@ -257,9 +254,7 @@ gedit_cmd_edit_copy (BonoboUIComponent *uic, gpointer user_data, const gchar* ve
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
 	
-	gedit_view_copy_clipboard (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	gedit_view_copy_clipboard (active_view); 
 }
 
 void 
@@ -271,8 +266,6 @@ gedit_cmd_edit_paste (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 	g_return_if_fail (active_view);
 	
 	gedit_view_paste_clipboard (active_view); 
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
 }
 
 void 
@@ -283,26 +276,18 @@ gedit_cmd_edit_clear (BonoboUIComponent *uic, gpointer user_data, const gchar* v
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
 	
-	gedit_view_delete_selection (active_view);
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	gedit_view_delete_selection (active_view); 
 }
 
 void
 gedit_cmd_edit_select_all (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	GeditDocument* active_doc;
 	GeditView* active_view;
 
 	active_view = GEDIT_VIEW (bonobo_mdi_get_active_view (BONOBO_MDI (gedit_mdi)));
 	g_return_if_fail (active_view);
-
-	active_doc = gedit_get_active_document ();
-	g_return_if_fail (active_doc);
-
-	gedit_document_set_selection (active_doc, 0, -1); 
-
-	gtk_widget_grab_focus (GTK_WIDGET (active_view));
+	
+	gedit_view_select_all (active_view); 
 }
 
 void 
@@ -335,46 +320,29 @@ gedit_cmd_search_find_again (BonoboUIComponent *uic, gpointer user_data, const g
 	last_searched_text = gedit_document_get_last_searched_text (doc);
 	if (last_searched_text != NULL)
 	{
-		gpointer data;
-		gboolean was_wrap_around;
-		gboolean found;
-
-		data = g_object_get_qdata (G_OBJECT (doc), gedit_was_wrap_around_quark ());
-		if (data == NULL)
-			was_wrap_around = TRUE;
-		else
-			was_wrap_around = GPOINTER_TO_BOOLEAN (data);
-
-		found = gedit_document_find_again (doc, TRUE);
-
-		if (!found && was_wrap_around)
-			found = gedit_document_find_again (doc, FALSE);
-
-		if (!found)
+		if (!gedit_document_find_again (doc))
 		{	
 			GtkWidget *message_dlg;
 
 			message_dlg = gtk_message_dialog_new (
-				GTK_WINDOW (gedit_get_active_window ()),
+				GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
 				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
-				_("The text \"%s\" was not found."), last_searched_text);
-
-			gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
-
-			gtk_window_set_resizable (GTK_WINDOW (message_dlg), FALSE);
-
+				_("The searched string has not been found."));
+			
 			gtk_dialog_run (GTK_DIALOG (message_dlg));
   			gtk_widget_destroy (message_dlg);
 		}
 		else
 			gedit_view_scroll_to_cursor (active_view);
 
-		g_free (last_searched_text);
 	}
 	else
+	{
+		g_free (last_searched_text);
 		gedit_dialog_find ();
+	}
 }
 
 
@@ -383,7 +351,10 @@ gedit_cmd_search_replace (BonoboUIComponent *uic, gpointer user_data, const gcha
 {
 	gedit_debug (DEBUG_COMMANDS, "");
 
+	TO_BE_IMPLEMENTED
+/*
 	gedit_dialog_replace ();
+*/
 }
 
 void 
@@ -397,124 +368,68 @@ gedit_cmd_search_goto_line (BonoboUIComponent *uic, gpointer user_data, const gc
 void
 gedit_cmd_settings_preferences (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	static GtkWidget *dlg = NULL;
+	GtkWidget *dlg;
+	gint ret;
 
-	gedit_debug (DEBUG_COMMANDS, "");
+	dlg = gedit_preferences_dialog_new (
+			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))));
 
-	if (dlg != NULL)
+	do
 	{
-		gtk_window_present (GTK_WINDOW (dlg));
-		gtk_window_set_transient_for (GTK_WINDOW (dlg),	
-					      GTK_WINDOW (gedit_get_active_window ()));
+		ret = gtk_dialog_run (GTK_DIALOG (dlg));
 
-		return;
-	}
+		switch (ret)
+		{
+			case GTK_RESPONSE_OK:
+				if (gedit_preferences_dialog_update_settings (GEDIT_PREFERENCES_DIALOG (dlg)))
+				{
+					gedit_mdi_update_ui_according_to_preferences (gedit_mdi);
+					gedit_prefs_save_settings ();
+				}
+
+				break;
+			case GTK_RESPONSE_HELP:
+				/* FIXME */
+				break;
+		}	
 		
-	dlg = gedit_preferences_dialog_new (GTK_WINDOW (gedit_get_active_window ()));
+	} while (ret == GTK_RESPONSE_HELP);
 
-	g_signal_connect (G_OBJECT (dlg), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &dlg);
-	
-	gtk_widget_show (dlg);
-}
-
-void
-gedit_cmd_documents_move_to_new_window (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
-{
-	GeditView *view;
-
-	gedit_debug (DEBUG_COMMANDS, "");
-
-	view = gedit_get_active_view ();
-	g_return_if_fail (view != NULL);
-
-	bonobo_mdi_move_view_to_new_window (BONOBO_MDI (gedit_mdi), GTK_WIDGET (view));
-}
-
-void 
-gedit_cmd_help_contents (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
-{
-	GError *error = NULL;
-
-	gedit_debug (DEBUG_COMMANDS, "");
-
-	gnome_help_display ("gedit.xml", NULL, &error);
-	
-	if (error != NULL)
-	{
-		g_warning (error->message);
-
-		g_error_free (error);
-	}
+	gtk_widget_destroy (dlg);
 }
 
 void 
 gedit_cmd_help_about (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname)
 {
-	static GtkWidget *about = NULL;
-	GdkPixbuf* pixbuf = NULL;
+	/* FIXME */
+	static const char *authors[] = { "Chema Celorio", 
+					 "Paolo Maggi", 
+					  NULL };
 	
-	gchar *authors[] = {
-		"Paolo Maggi <maggi@athena.polito.it>",
-		"Chema Celorio <chema@ximian.com>", 
-		"James Willcox <jwillcox@cs.indiana.edu>",
-		"Federico Mena Quintero <federico@ximian.com>",
-		NULL
-	};
+	static const char *documentors[] = { "Eric Baudais", NULL };
 	
-	gchar *documenters[] = {
-		"Sun GNOME Documentation Team <gdocteam@sun.com>",
-		"Eric Baudais <baudais@okstate.edu>",
-		NULL
-	};
+	GtkWidget *about_box = NULL;
 	
-	gchar *translator_credits = _("translator_credits");
-
 	gedit_debug (DEBUG_COMMANDS, "");
-
-	if (about != NULL)
-	{
-		gdk_window_show (about->window);
-		gdk_window_raise (about->window);
-		return;
-	}
 	
-	pixbuf = gdk_pixbuf_new_from_file ( GNOME_ICONDIR "/gedit-logo.png", NULL);
-	if (pixbuf != NULL)
-	{
-		GdkPixbuf* temp_pixbuf = NULL;
+	g_print ("****** About box in gedit2 is completely broken. Please don't report bugs about it.\n");
 
-		temp_pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
-					 gdk_pixbuf_get_width (pixbuf),
-					 gdk_pixbuf_get_height (pixbuf),
-					 GDK_INTERP_HYPER);
-		g_object_unref (pixbuf);
+	about_box = gnome_about_new ("gedit",
+				VERSION,
+				"(c) 1998-2000 Alex Robert and Ewan Lawrence\n"
+				"(c) 2000-2001 Chema Celorio and Paolo Maggi",
+				"gedit is a small and lightweight text editor for GNOME",
+				authors,
+				documentors,
+				NULL /*"Translation credits"*/,
+				NULL /* logo pixbuf */);
 
-		pixbuf = temp_pixbuf;
-	}
 
-	about = gnome_about_new (_("gedit"), VERSION,
-				_("(C) 1998-2000 Evan Lawrence and Alex Robert\n"
-				  "(C) 2000-2002 Chema Celorio and Paolo Maggi"),
-				_("gedit is a small and lightweight text editor for Gnome"),
-				(const char **)authors,
-				(const char **)documenters,
-				strcmp (translator_credits, "translator_credits") != 0 ? 
-					(const char *)translator_credits : NULL,
-				pixbuf);
+	gtk_window_set_transient_for (GTK_WINDOW (about_box),
+			      GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))));
 
-	gtk_window_set_transient_for (GTK_WINDOW (about),
-			GTK_WINDOW (gedit_get_active_window ()));
+	gtk_dialog_run (GTK_DIALOG (about_box));
 
-	gtk_window_set_destroy_with_parent (GTK_WINDOW (about), TRUE);
-
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
-	
-	g_signal_connect (G_OBJECT (about), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &about);
-	
-	gtk_widget_show (about);
+	gtk_widget_destroy (about_box);
 }
 
-
