@@ -30,12 +30,10 @@
 
 #include <glib.h>
 #include <stdlib.h>
-#include <string.h>
 
 #include "gedit-undo-manager.h"
 #include "gedit-debug.h"
-#include "gedit-prefs-manager.h"
-#include "gedit-marshal.h"
+#include "gedit-prefs.h"
 
 typedef struct _GeditUndoAction  		GeditUndoAction;
 typedef struct _GeditUndoInsertAction		GeditUndoInsertAction;
@@ -43,7 +41,7 @@ typedef struct _GeditUndoDeleteAction		GeditUndoDeleteAction;
 
 typedef enum {
 	GEDIT_UNDO_ACTION_INSERT,
-	GEDIT_UNDO_ACTION_DELETE
+	GEDIT_UNDO_ACTION_DELETE,
 } GeditUndoActionType;
 
 /* 
@@ -56,7 +54,6 @@ struct _GeditUndoInsertAction
 	gint   pos; 
 	gchar *text;
 	gint   length;
-	gint   chars;
 };
 
 struct _GeditUndoDeleteAction
@@ -93,8 +90,6 @@ struct _GeditUndoManagerPrivate
 	gboolean	can_redo;
 
 	gint		running_not_undoable_actions;
-
-	gint		num_of_groups;
 };
 
 enum {
@@ -124,6 +119,7 @@ static void gedit_undo_manager_check_list_size 		(GeditUndoManager *um);
 
 static gboolean gedit_undo_manager_merge_action 	(GeditUndoManager *um, 
 		                                         GeditUndoAction *undo_action);
+static guint gedit_undo_manager_get_number_of_groups 	(GeditUndoManager *um);
 
 static GObjectClass 	*parent_class 				= NULL;
 static guint 		undo_manager_signals [LAST_SIGNAL] 	= { 0 };
@@ -175,7 +171,7 @@ gedit_undo_manager_class_init (GeditUndoManagerClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditUndoManagerClass, can_undo),
 			      NULL, NULL,
-			      gedit_marshal_VOID__BOOLEAN,
+			      gtk_marshal_VOID__BOOLEAN,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
@@ -186,7 +182,7 @@ gedit_undo_manager_class_init (GeditUndoManagerClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GeditUndoManagerClass, can_redo),
 			      NULL, NULL,
-			      gedit_marshal_VOID__BOOLEAN,
+			      gtk_marshal_VOID__BOOLEAN,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
@@ -207,8 +203,6 @@ gedit_undo_manager_init (GeditUndoManager *um)
 	um->priv->can_redo = FALSE;
 
 	um->priv->running_not_undoable_actions = 0;
-
-	um->priv->num_of_groups = 0;
 }
 
 static void
@@ -282,8 +276,7 @@ gedit_undo_manager_new (GeditDocument* document)
 	return um;
 }
 
-void 
-gedit_undo_manager_begin_not_undoable_action (GeditUndoManager *um)
+void gedit_undo_manager_begin_not_undoable_action (GeditUndoManager *um)
 {
 	gedit_debug (DEBUG_UNDO, "");
 
@@ -293,50 +286,19 @@ gedit_undo_manager_begin_not_undoable_action (GeditUndoManager *um)
 	++um->priv->running_not_undoable_actions;
 }
 
-static void 
-gedit_undo_manager_end_not_undoable_action_internal (GeditUndoManager *um)
+void gedit_undo_manager_end_not_undoable_action (GeditUndoManager *um)
 {
 	gedit_debug (DEBUG_UNDO, "");
 
 	g_return_if_fail (GEDIT_IS_UNDO_MANAGER (um));
 	g_return_if_fail (um->priv != NULL);
 
-	g_return_if_fail (um->priv->running_not_undoable_actions > 0);
-	
-	--um->priv->running_not_undoable_actions;
+	if (um->priv->running_not_undoable_actions <= 0)
+		um->priv->running_not_undoable_actions = 0;
+	else
+		--um->priv->running_not_undoable_actions;
 }
 
-void 
-gedit_undo_manager_end_not_undoable_action (GeditUndoManager *um)
-{
-	gedit_debug (DEBUG_UNDO, "");
-
-	g_return_if_fail (GEDIT_IS_UNDO_MANAGER (um));
-	g_return_if_fail (um->priv != NULL);
-
-	gedit_undo_manager_end_not_undoable_action_internal (um);
-
-	if (um->priv->running_not_undoable_actions == 0)
-	{	
-		gedit_undo_manager_free_action_list (um);
-	
-		um->priv->next_redo = -1;	
-
-		if (um->priv->can_undo)
-		{
-			um->priv->can_undo = FALSE;
-			g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_UNDO], 0, FALSE);
-		}
-
-		if (um->priv->can_redo)
-		{
-			um->priv->can_redo = FALSE;
-			g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_REDO], 0, FALSE);
-		}
-	}
-}
-
-
 gboolean
 gedit_undo_manager_can_undo (const GeditUndoManager *um)
 {
@@ -382,27 +344,19 @@ gedit_undo_manager_undo (GeditUndoManager *um)
 		switch (undo_action->action_type)
 		{
 			case GEDIT_UNDO_ACTION_DELETE:
-				gedit_document_set_cursor (
-					um->priv->document, 
-					undo_action->action.delete.start);
-
 				gedit_document_insert_text (
 					um->priv->document, 
 					undo_action->action.delete.start, 
 					undo_action->action.delete.text,
-					strlen (undo_action->action.delete.text));
-
-								break;
+					abs (undo_action->action.delete.end - 
+						undo_action->action.delete.start));
+				break;
 			case GEDIT_UNDO_ACTION_INSERT:
 				gedit_document_delete_text (
 					um->priv->document, 
 					undo_action->action.insert.pos, 
 					undo_action->action.insert.pos + 
-						undo_action->action.insert.chars); 
-
-				gedit_document_set_cursor (
-					um->priv->document, 
-					undo_action->action.insert.pos);
+						undo_action->action.insert.length); 
 				break;
 
 			default:
@@ -412,7 +366,7 @@ gedit_undo_manager_undo (GeditUndoManager *um)
 
 	} while (undo_action->order_in_group > 1);
 
-	gedit_undo_manager_end_not_undoable_action_internal (um);
+	gedit_undo_manager_end_not_undoable_action (um);
 	
 	if (!um->priv->can_redo)
 	{
@@ -420,7 +374,7 @@ gedit_undo_manager_undo (GeditUndoManager *um)
 		g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_REDO], 0, TRUE);
 	}
 
-	if (um->priv->next_redo >= (gint)(g_list_length (um->priv->actions) - 1))
+	if (um->priv->next_redo >= (g_list_length (um->priv->actions) - 1))
 	{
 		um->priv->can_undo = FALSE;
 		g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_UNDO], 0, FALSE);
@@ -452,18 +406,9 @@ gedit_undo_manager_redo (GeditUndoManager *um)
 					um->priv->document, 
 					undo_action->action.delete.start, 
 					undo_action->action.delete.end); 
-
-				gedit_document_set_cursor (
-					um->priv->document,
-					undo_action->action.delete.start);
-
 				break;
 				
 			case GEDIT_UNDO_ACTION_INSERT:
-				gedit_document_set_cursor (
-					um->priv->document,
-					undo_action->action.insert.pos);
-
 				gedit_document_insert_text (
 					um->priv->document, 
 					undo_action->action.insert.pos, 
@@ -486,7 +431,7 @@ gedit_undo_manager_redo (GeditUndoManager *um)
 		
 	} while ((undo_action != NULL) && (undo_action->order_in_group > 1));
 
-	gedit_undo_manager_end_not_undoable_action_internal (um);
+	gedit_undo_manager_end_not_undoable_action (um);
 
 	if (um->priv->next_redo < 0)
 	{
@@ -522,20 +467,9 @@ gedit_undo_manager_free_action_list (GeditUndoManager *um)
 	{
 		GeditUndoAction *undo_action = 
 			(GeditUndoAction *)(g_list_nth_data (um->priv->actions, n));
-
-		gedit_debug (DEBUG_UNDO, "Free action (type %s) %d/%d", 
-				(undo_action->action_type == GEDIT_UNDO_ACTION_INSERT) ? "insert":
-				"delete", n, len);
-
+	
 		if (undo_action->action_type == GEDIT_UNDO_ACTION_INSERT)
-			g_free (undo_action->action.insert.text);
-		else if (undo_action->action_type == GEDIT_UNDO_ACTION_DELETE)
-			g_free (undo_action->action.delete.text);
-		else
-			g_return_if_fail (FALSE);
-
-		if (undo_action->order_in_group == 1)
-			--um->priv->num_of_groups;
+			g_free (undo_action->action.insert.text);	
 
 		g_free (undo_action);
 	}
@@ -555,17 +489,13 @@ gedit_undo_manager_insert_text_handler (GtkTextBuffer *buffer, GtkTextIter *pos,
 	if (um->priv->running_not_undoable_actions > 0)
 		return;
 
-	g_return_if_fail (strlen (text) == (guint)length);
-	
 	undo_action.action_type = GEDIT_UNDO_ACTION_INSERT;
 
 	undo_action.action.insert.pos    = gtk_text_iter_get_offset (pos);
-	undo_action.action.insert.text   = (gchar*) text;
+	undo_action.action.insert.text   = g_strdup (text);
 	undo_action.action.insert.length = length;
-	undo_action.action.insert.chars  = g_utf8_strlen (text, length);
-
-	if ((undo_action.action.insert.chars > 1) || (g_utf8_get_char (text) == '\n'))
 
+	if ((length > 1) || (*text == '\n'))
 	       	undo_action.mergeable = FALSE;
 	else
 		undo_action.mergeable = TRUE;
@@ -586,8 +516,6 @@ gedit_undo_manager_delete_range_handler (GtkTextBuffer *buffer, GtkTextIter *sta
 
 	undo_action.action_type = GEDIT_UNDO_ACTION_DELETE;
 
-	gtk_text_iter_order (start, end);
-
 	undo_action.action.delete.start  = gtk_text_iter_get_offset (start);
 	undo_action.action.delete.end    = gtk_text_iter_get_offset (end);
 
@@ -595,22 +523,13 @@ gedit_undo_manager_delete_range_handler (GtkTextBuffer *buffer, GtkTextIter *sta
 						GEDIT_DOCUMENT (buffer),
 						undo_action.action.delete.start,
 						undo_action.action.delete.end);
-
-	if (((undo_action.action.delete.end - undo_action.action.delete.start) > 1) ||
-	     (g_utf8_get_char (undo_action.action.delete.text  ) == '\n'))
+	
+	if ((undo_action.action.delete.end - undo_action.action.delete.start) > 1)
 	       	undo_action.mergeable = FALSE;
 	else
 		undo_action.mergeable = TRUE;
 
-	gedit_debug (DEBUG_UNDO, "START: %d", undo_action.action.delete.start);
-	gedit_debug (DEBUG_UNDO, "END: %d", undo_action.action.delete.end);
-	gedit_debug (DEBUG_UNDO, "TEXT: %s", undo_action.action.delete.text);
-
 	gedit_undo_manager_add_action (um, undo_action);
-
-	g_free (undo_action.action.delete.text);
-
-	gedit_debug (DEBUG_UNDO, "DONE");
 }
 
 static void 
@@ -657,22 +576,9 @@ gedit_undo_manager_add_action (GeditUndoManager *um, GeditUndoAction undo_action
 	{
 		action = g_new (GeditUndoAction, 1);
 		*action = undo_action;
-
-		if (action->action_type == GEDIT_UNDO_ACTION_INSERT)
-			action->action.insert.text = g_strdup (undo_action.action.insert.text);
-		else if (action->action_type == GEDIT_UNDO_ACTION_DELETE)
-			action->action.delete.text = g_strdup (undo_action.action.delete.text); 
-		else
-		{
-			g_free (action);
-			g_return_if_fail (FALSE);
-		}
-		
+	
 		++um->priv->actions_in_current_group;
 		action->order_in_group = um->priv->actions_in_current_group;
-
-		if (action->order_in_group == 1)
-			++um->priv->num_of_groups;
 	
 		um->priv->actions = g_list_prepend (um->priv->actions, action);
 	}
@@ -691,7 +597,6 @@ gedit_undo_manager_add_action (GeditUndoManager *um, GeditUndoAction undo_action
 		g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_REDO], 0, FALSE);
 	}
 
-	gedit_debug (DEBUG_UNDO, "DONE");
 }
 
 static void 
@@ -719,9 +624,6 @@ gedit_undo_manager_free_first_n_actions (GeditUndoManager *um, gint n)
 		else
 			g_return_if_fail (FALSE);
 
-		if (undo_action->order_in_group == 1)
-			--um->priv->num_of_groups;
-
 		g_free (undo_action);
 
 		um->priv->actions = g_list_delete_link (um->priv->actions, um->priv->actions);
@@ -734,19 +636,15 @@ gedit_undo_manager_free_first_n_actions (GeditUndoManager *um, gint n)
 static void 
 gedit_undo_manager_check_list_size (GeditUndoManager *um)
 {
-	gint undo_levels;
-	
 	gedit_debug (DEBUG_UNDO, "TODO");
 
 	g_return_if_fail (GEDIT_IS_UNDO_MANAGER (um));
 	g_return_if_fail (um->priv != NULL);
 	
-	undo_levels = gedit_prefs_manager_get_undo_actions_limit ();
-	
-	if (undo_levels < 1)
+	if (settings->undo_levels < 1)
 		return;
 
-	if (um->priv->num_of_groups > undo_levels)
+	if (gedit_undo_manager_get_number_of_groups (um) > settings->undo_levels)
 	{
 		GeditUndoAction *undo_action;
 		GList* last;
@@ -763,9 +661,6 @@ gedit_undo_manager_check_list_size (GeditUndoManager *um)
 			else
 				g_return_if_fail (FALSE);
 
-			if (undo_action->order_in_group == 1)
-				--um->priv->num_of_groups;
-
 			g_free (undo_action);
 
 			um->priv->actions = g_list_delete_link (um->priv->actions, last);
@@ -775,7 +670,7 @@ gedit_undo_manager_check_list_size (GeditUndoManager *um)
 			undo_action = (GeditUndoAction*) last->data;
 
 		} while ((undo_action->order_in_group > 1) || 
-			 (um->priv->num_of_groups > undo_levels));
+			 (gedit_undo_manager_get_number_of_groups (um) > settings->undo_levels));
 	}	
 }
 
@@ -816,7 +711,7 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 	}
 
 	if (undo_action->action_type == GEDIT_UNDO_ACTION_DELETE)
-	{				
+	{
 		if ((last_action->action.delete.start != undo_action->action.delete.start) &&
 		    (last_action->action.delete.start != undo_action->action.delete.end))
 		{
@@ -829,15 +724,12 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 			gchar *str;
 			
 #define L  (last_action->action.delete.end - last_action->action.delete.start - 1)
-#define g_utf8_get_char_at(p,i) g_utf8_get_char(g_utf8_offset_to_pointer((p),(i)))
-		
-			gedit_debug (DEBUG_UNDO, "L = %d", L);
-
+			
 			/* Deleted with the delete key */
-			if ((g_utf8_get_char (undo_action->action.delete.text) != ' ') &&
-			    (g_utf8_get_char (undo_action->action.delete.text) != '\t') &&
-                            ((g_utf8_get_char_at (last_action->action.delete.text, L) == ' ') ||
-			     (g_utf8_get_char_at (last_action->action.delete.text, L)  == '\t')))
+			if ((undo_action->action.delete.text [0] != ' ') &&
+			    (undo_action->action.delete.text [0] != '\t') &&
+                            ((last_action->action.delete.text [L] == ' ') ||
+			     (last_action->action.delete.text [L] == '\t')))
 			{
 				last_action->mergeable = FALSE;
 				return FALSE;
@@ -847,7 +739,7 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 				undo_action->action.delete.text);
 			
 			g_free (last_action->action.delete.text);
-			last_action->action.delete.end += (undo_action->action.delete.end - undo_action->action.delete.start);
+			last_action->action.delete.end += 1;
 			last_action->action.delete.text = str;
 		}
 		else
@@ -855,10 +747,10 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 			gchar *str;
 			
 			/* Deleted with the backspace key */
-			if ((g_utf8_get_char (undo_action->action.delete.text) != ' ') &&
-			    (g_utf8_get_char (undo_action->action.delete.text) != '\t') &&
-                            ((g_utf8_get_char (last_action->action.delete.text) == ' ') ||
-			     (g_utf8_get_char (last_action->action.delete.text) == '\t')))
+			if ((undo_action->action.delete.text [0] != ' ') &&
+			    (undo_action->action.delete.text [0] != '\t') &&
+                            ((last_action->action.delete.text [0] == ' ') ||
+			     (last_action->action.delete.text [0] == '\t')))
 			{
 				last_action->mergeable = FALSE;
 				return FALSE;
@@ -875,15 +767,13 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 	else if (undo_action->action_type == GEDIT_UNDO_ACTION_INSERT)
 	{
 		gchar* str;
-		
-#define I (last_action->action.insert.chars - 1)
-		
+
 		if ((undo_action->action.insert.pos != 
-		     	(last_action->action.insert.pos + last_action->action.insert.chars)) ||
-		    ((g_utf8_get_char (undo_action->action.insert.text) != ' ') &&
-		      (g_utf8_get_char (undo_action->action.insert.text) != '\t') &&
-		     ((g_utf8_get_char_at (last_action->action.insert.text, I) == ' ') ||
-		      (g_utf8_get_char_at (last_action->action.insert.text, I) == '\t')))
+		     	(last_action->action.insert.pos + last_action->action.insert.length)) ||
+		    ((undo_action->action.insert.text [0] != ' ') &&
+		      (undo_action->action.insert.text [0] != '\t') &&
+		     ((last_action->action.insert.text [last_action->action.insert.length - 1] == ' ') ||
+		      (last_action->action.insert.text [last_action->action.insert.length - 1] == '\t')))
 		   )
 		{
 			last_action->mergeable = FALSE;
@@ -896,8 +786,6 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 		g_free (last_action->action.insert.text);
 		last_action->action.insert.length += undo_action->action.insert.length;
 		last_action->action.insert.text = str;
-		last_action->action.insert.chars += undo_action->action.insert.chars;
-
 	}
 	else
 		g_warning ("Unknown action inside undo merge encountered");	
@@ -905,4 +793,21 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 	return TRUE;
 }
 
+static guint
+gedit_undo_manager_get_number_of_groups (GeditUndoManager *um)
+{
+	GList *list = um->priv->actions;
+  	guint num = 0;
+
+	gedit_debug (DEBUG_UNDO, "");
+  
+  	while (list)
+    	{
+      		if (((GeditUndoAction* ) list->data)->order_in_group == 1)
+			++num;
+      		list = list->next;
+    	}
+  
+  	return num;
+}
 
